/*
 * Copyright (c) 2001 Massachusetts Institute of Technology.
 *
 * This file is part of bgptools. 
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `COPYING' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file AUTHORS which should have accompanied this software 
 * for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the AUTHORS file.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */


/*
 *     Nick Feamster (feamster@lcs.mit.edu)
 *     MIT Lab for Computer Science
 *     November 2001
 */

#ifndef _BGP_DUMP_H
#define _BGP_DUMP_H

#include <sys/types.h>
#include <stdio.h>
#include <string.h>
#include <netinet/in.h>
#include <unistd.h>

#include "config.h"
#include "common.h"
#include "stats.h"

typedef struct prefix_s {
  u_int32_t pfx;
  u_int32_t len;
} prefix_s;

typedef struct table_entry {
  u_int16_t view, seq, peer_as;
  u_int32_t uptime;
  char tpfx[sizeof("xxx.xxx.xxx.xxx./xx")]; 
  char peer_ip[sizeof("xxx.xxx.xxx.xxx./xx")];
  char tlen, tstat;
} table_entry;

typedef struct update {
  unsigned long time;
  u_int16_t dump_type;
  u_int16_t dump_subtype;
  u_int16_t src_as;
  u_int16_t dst_as;

  u_int16_t if_ind;
  u_int16_t addr_fam;
  u_int16_t msg_length;
  u_int16_t msg_type;

  char ad_type;

  struct prefix_s *prefix_;
  struct prefix_s *prefix_wd_;

  char *prefix_str;
  
  int num_prefixes;
  int num_prefixes_wd;
  
  char origin;
  char as_path[256];
  u_int16_t origin_as;

  u_int32_t next_hop;
  u_int16_t med;
  u_int16_t localpref;

  char *community;
  u_int16_t aggregator;
  u_int32_t aggregator_ip;

} update;

/* MRT and RADIX compatible packet dump values. */
/* type value */

enum MRT_MSG_TYPES {
   MSG_NULL,
   MSG_START,                   /* sender is starting up */
   MSG_DIE,                     /* receiver should shut down */
   MSG_I_AM_DEAD,               /* sender is shutting down */
   MSG_PEER_DOWN,               /* sender's peer is down */
   MSG_PROTOCOL_BGP,            /* msg is a BGP packet */
   MSG_PROTOCOL_RIP,            /* msg is a RIP packet */
   MSG_PROTOCOL_IDRP,           /* msg is an IDRP packet */
   MSG_PROTOCOL_RIPNG,          /* msg is a RIPNG packet */
   MSG_PROTOCOL_BGP4PLUS,       /* msg is a BGP4+ packet */
   MSG_PROTOCOL_BGP4PLUS_01,    /* msg is a BGP4+ (draft 01) packet */
   MSG_PROTOCOL_OSPF,           /* msg is an OSPF packet */
   MSG_TABLE_DUMP               /* routing table dump */
};

#define MSG_PROTOCOL_BGP4MP  16
/* subtype value */
#define BGP4MP_STATE_CHANGE   0
#define BGP4MP_MESSAGE        1
#define BGP4MP_ENTRY          2
#define BGP4MP_SNAPSHOT       3

#define BGP_DUMP_HEADER_SIZE 12


#define BGP_OPEN              1
#define BGP_UPDATE            2
#define BGP_NOTIFICATION      3
#define BGP_KEEPALIVE         4

#define ORIGIN                1
#define AS_PATH               2
#define NEXT_HOP              3
#define MULTI_EXIT_DISC       4
#define LOCAL_PREF            5
#define ATOMIC_AGGREGATE      6
#define AGGREGATOR            7

#define IGP                   0
#define EGP                   1
#define INCOMPLETE            2

#define BGP_ADV               0
#define BGP_WITHDRAWAL        1

const char *intoa(u_int32_t addr);
void read_record(FILE *fp, struct update *upd, struct table_entry *te=NULL);
void read_record(char *pkt, int plen, struct update *upd);
void read_update_file(char *filename);
void parse_file_mrt(char *filename, struct opts *opt, update_stats *stats);


void print_mrt_format(update *upd, FILE *fd);
void print_long_format(update *upd, FILE *fd);

#endif /* _BGP_DUMP_H */
