#include "dbquery.h"
#ifdef __USE_SQLPLUS__


dbquery::dbquery(char *db, cchar *host=DB_HOST, cchar *user="", cchar *passwd="") {
  this->db_file = db;
  this->host = host;
  fprintf(stderr, "connection to %s at %s...", db, host);
  this->c = new Connection(db, host, user, passwd);
  fprintf(stderr, "succeeded.\n");
}

dbquery::~dbquery() {
  delete c;
}

Result
dbquery::make_query(char *querystr) {
  Query query = c->query();
  query << querystr;
  return query.store();
}

void 
dbquery::multihomed_prefixes() {

  map<string, string, ltstr > orig_hash;
//    hash_map<const char*, char*, hash<const char*>, eqstr > orig_hash;

  Query query = c->query();
  char querystr[1024];
  
  sprintf(querystr, "select distinct prefix,origin_as,aspath from %s.%s where (type='A') order by prefix;", this->db_file, UPDATE_TABLE);
  query << querystr;

  Result res = query.store();
  fprintf(stdout, "found %d records...\n", res.size());
  int count=0;


  Result::iterator i;
  Row row;
  orig_hash.clear();

  string last_prefix, last_origin;

  for (i=res.begin(); i!=res.end(); i++) {
    row = *i;

    string prefix = (string)row["prefix"];
    string origin = (string)row["origin_as"];
    
    if (orig_hash[prefix].length() && 
	!strcmp(last_prefix.c_str(),prefix.c_str()) &&
	strcmp(last_origin.c_str(),origin.c_str())) {

      if (!strstr(orig_hash[prefix].c_str(),origin.c_str())) {
	orig_hash[prefix] = orig_hash[prefix] + "," + origin;
      }
  
    } else {

      if (strlen(last_prefix.c_str()) &&
	  strstr(orig_hash[last_prefix].c_str(), ",") &&
	  strcmp(last_prefix.c_str(),prefix.c_str())) {
  	count++;
      	fprintf(stdout, "%s => %s\n", prefix.c_str(), orig_hash[last_prefix].c_str());
      }
	
      
      orig_hash[prefix] = origin;
    }
    
    last_prefix = prefix;
    last_origin = origin;

  }
  fprintf(stdout, "*** %d prefixes with multiple origins\n", count);
}



void
dbquery::withdrawal_intervals() {
  char querystr[80];
  map<string, unsigned long, ltstr> last_wd;
  map<string, int, ltstr> is_wd;
  map<string, int, ltstr> count_wd;
  map<string, int, ltstr>::const_iterator k;

  
  hash_map<unsigned long, unsigned long, hash<unsigned long>, equal_to<unsigned long> > downtimes;

  sprintf(querystr, "select prefix,type,unixtime from %s.%s order by unixtime;", this->db_file, UPDATE_TABLE);
  Query query = c->query();
  query << querystr;
  Result res = query.store();


  fprintf(stdout, "found %d records...\n", res.size());

  Result::iterator i;
  Row row;
  int count=0;

  for (i=res.begin(); i!=res.end(); i++) {
    row = *i;


    string prefix = (string)row["prefix"];
    char type = (signed char)(*(row.raw_data(1)));
    string utimestr = (string)row["unixtime"];
    int unixtime = atoi((char*)(utimestr.c_str()));

    if (type == 'W') {
      last_wd[prefix] = unixtime;
      is_wd[prefix] = 1;

      count_wd[prefix] = count_wd[prefix]+1;
      
    } else if (type == 'A') {
      if (last_wd[prefix] > 0 && is_wd[prefix]==1) {
//  	fprintf(stderr, "A %s %d\n", prefix, last_wd[prefix]);
	is_wd[prefix]=0;
	int downtime = unixtime-last_wd[prefix];
	downtimes[downtime] = downtimes[downtime]+1;
      }
    }

    // track progress...
    count++;
    if (!(count % 10000)) { fprintf(stderr, "%d\n", count); }
  }

  FILE *fp = fopen ("wd.dat", "w");
  hash_map<unsigned long, unsigned long, hash<unsigned long>, equal_to<unsigned long> >::const_iterator j;
  for (j=downtimes.begin(); j!=downtimes.end(); j++) {
    fprintf(fp, "%d\t%d\n", (*j).first, (*j).second);
  }
  fclose(fp);

  FILE *fp2 = fopen("count.dat", "w");
  for (k=count_wd.begin(); k!=count_wd.end(); k++) {
    fprintf(fp2, "%s\t%d\n", (*k).first.c_str(), (*k).second);
  }
  fclose(fp2);

  
}

#endif
